@@ -12,7 +12,7 @@
 ##############################################################################
 """ Unit tests for CalendarTool module.
 
-$Id: test_Calendar.py 37158 2005-07-16 02:30:32Z tseaver $
+$Id: test_Calendar.py 41262 2006-01-11 13:50:36Z yuppie $
 """
 
 import unittest
@@ -40,6 +40,7 @@ try:
 except ImportError:
     # BBB: for Zope 2.7
     from Products.CMFCore.utils import transaction
+from Products.CMFCore import Skinnable
 
 
 class CalendarTests(unittest.TestCase):
@@ -86,6 +87,7 @@ class CalendarRequestTests(unittest.TestCase, WarningInterceptor):
 
     def setUp(self):
         self._trap_warning_output()
+        self._oldSkindata = Skinnable.SKINDATA.copy()
         transaction.begin()
 
         app = self.app = makerequest(Zope2.app())
@@ -120,9 +122,11 @@ class CalendarRequestTests(unittest.TestCase, WarningInterceptor):
                               app.session_data_manager.getSessionData )
 
     def tearDown(self):
+        self.app.REQUEST.close()
         noSecurityManager()
         transaction.abort()
         self.app._p_jar.close()
+        Skinnable.SKINDATA = self._oldSkindata
         self._free_warning_output()
 
     def _testURL(self,url,params=None):
@@ -556,6 +560,40 @@ class CalendarRequestTests(unittest.TestCase, WarningInterceptor):
 
         self.assertEqual(len(self.Site.portal_calendar.getEventsForThisDay(DateTime('2002/05/01'))), 2)
 
+    def test_EventEndingMidnight(self):
+        # Events ending exactly at midnight should not be shown for the day
+        # after (see http://www.zope.org/Collectors/CMF/246)
+        cal = self.Site.portal_calendar
+        the_day = DateTime('2002/05/01')
+        day_after = DateTime('2002/05/02')
+
+        self.Site.invokeFactory( 'Event'
+                               , id='party'
+                               , start_date=the_day
+                               , end_date=day_after
+                               )
+        self.Site.portal_workflow.doActionFor(self.Site.party, 'publish')
+
+        # One entry should be present for the day of the event
+        self.assertEqual(len(cal.getEventsForThisDay(the_day)), 1)
+
+        # No entry should be present for the day after
+        self.assertEqual(len(cal.getEventsForThisDay(day_after)), 0)
+
+        # First week of May 2002
+        data = [
+               {'day': 1, 'event': 1, 'eventslist':[{'start': '00:00:00', 'end': '23:59:59', 'title': 'party'}]},
+               {'day': 2, 'event': 0, 'eventslist':[]},
+               {'day': 3, 'event': 0, 'eventslist':[]},
+               {'day': 4, 'event': 0, 'eventslist':[]},
+               {'day': 5, 'event': 0, 'eventslist':[]},
+               {'day': 6, 'event': 0, 'eventslist':[]},
+               {'day': 7, 'event': 0, 'eventslist':[]},
+               ]
+
+        events = self.Site.portal_calendar.catalog_getevents(2002, 5)
+        self.assertEqual([events[e] for e in range(1, 8)], data)
+
 
 def test_suite():
     return unittest.TestSuite((
